"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const download = require("download");
const fs = require("fs");
const os = require("os");
const path = require("path");
const copy = require("recursive-copy");
const util_1 = require("../util");
const conf_1 = require("./conf");
const logger_1 = require("./logger");
const { resolve } = path;
exports.default = {
    async install({ pkgs = [], isGlobal = false, isDevDep = true, dir = process.cwd(), }) {
        if (pkgs && pkgs.length) {
            const args = ['i', ...pkgs];
            if (isGlobal) {
                args.push('-g');
            }
            if (isDevDep) {
                args.push('-D');
            }
            logger_1.debug(`cwd: ${dir}`);
            logger_1.debug(`npm ${args.join(' ')}`);
            await util_1.default.asyncSpawn('npm', args, {
                cwd: dir
            });
        }
        else {
            await util_1.default.asyncSpawn('npm', ['i'], {
                cwd: dir
            });
        }
    },
    async getInfo(pkg) {
        try {
            const { stdout } = await util_1.default.asyncExec(`npm view ${pkg} -json`);
            return JSON.parse(stdout);
        }
        catch (e) {
            if (e) { }
            return null;
        }
    },
    async pull(image, forceUpdate = false) {
        const imageDir = resolve(conf_1.default.npmPkgDir, image);
        if (!fs.existsSync(imageDir) || forceUpdate) {
            // @ts-ignore
            // tslint:disable-next-line:triple-equals
            if (os.platform() == 'win32') {
                await util_1.default.asyncExec(`rm -rf ${imageDir} && mkdir ${imageDir}`);
            }
            else {
                await util_1.default.asyncExec(`rm -rf ${imageDir} && mkdir -p ${imageDir}`);
            }
            const json = await this.getInfo(image);
            await download(json.dist.tarball, imageDir, {
                extract: true,
            });
            await util_1.default.asyncExec(`cd ${imageDir} && mv -f package/* . && rm -rf package`);
        }
    },
    async copy(mappings) {
        for (const mapping of mappings) {
            await copy(path.resolve(mapping.from), mapping.to, {
                overwrite: true,
                dot: true,
            });
        }
    },
};
